/***************************************************************************
 *   Copyright (C) 2004 by Thomas Nagy                                     *
 *   tnagy256@yahoo.fr                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <stdlib.h>

#include <qpainter.h>
#include <qcolor.h>
#include <qpixmap.h>

#include <kdebug.h>

#include "../datastruct/DDataItemBase.h"
#include "../datastruct/DDataControl.h"
#include "../itemconfig/ItemCfg.h"
#include "DCanvasView.h"
#include "DCanvasItemBase.h"
#include "DCanvasLink.h"
#include "DCanvasPos.h"

static const int itemz = 5;
static const int linkz = 4;

DCanvasItemBase::DCanvasItemBase(QWidget * parent, int id) : 
QCanvasRectangle(((QCanvasView*) parent)->canvas()), DGuiItem( parent, id)
{
	m_link = new DCanvasLink( canvas() );
	m_link->setZ(linkz);
	m_link->show();

	m_pos = new DCanvasPos( canvas(), this );
	m_pos->hide();
	m_pos->setZ(itemz);

	m_canvasview = static_cast<DCanvasView*> (parent);
	setSize(50, 50);
	m_color = QColor(random()%255, random()%30, random()%255);
	setSelected(false);
	setDisplayIdx(false);

	m_type = DItem::Unknown;
}

DCanvasItemBase::~DCanvasItemBase()
{
	hide();
	m_link->hide();
	m_pos->hide();

	delete m_link;
	delete m_pos;
}

void DCanvasItemBase::setSelected(bool b)
{
	// kdWarning()<<"DCanvasItemBase::setSelected - "<<Id()<<" "<<b<<endl;
	if (b)
		setZ(itemz+1);
	else
		setZ(itemz);
	QCanvasRectangle::setSelected(b);
}

void DCanvasItemBase::mouseReleaseEvent(QMouseEvent *me)
{
	setZ(itemz);
	moving = false;
}

void DCanvasItemBase::setOffset(const QPoint & p)
{
	m_old.setX( (int)x() );
	m_old.setY( (int)y() );

	m_pressOffset.setX( p.x() - m_old.x() );
	m_pressOffset.setY( p.y() - m_old.y() );
}

void DCanvasItemBase::mouseMoveEvent(QMouseEvent *me)
{
        if (!isSelected())
                setSelected(true);

	// TODO : cleanup
        int newX = 0, newY = 0;
        int moveX, moveY;
        int canvasW = canvas()->width();
        int canvasH =  canvas()->height();

        moveX = (int)me->x();
        moveY = (int)me->y();

        newX = moveX - m_pressOffset.x();
        newY = moveY - m_pressOffset.y();

        newX = newX<0 ? 0 : newX;
        newY = newY<0 ? 0 : newY;

        newX = newX+width()>canvasW ? canvasW - width() : newX;
        newY = newY+height()>canvasH ? canvasH - height() : newY;

        m_old.setX( newX );
        m_old.setY( newY );

        move(newX, newY);

        //adjustbranches();
}

void DCanvasItemBase::move(double xx, double yy)
{
        QCanvasItem::move(xx, yy);
        m_pos->move( x() - (3*RAY)/4, y()+RAY/2 );

        if (!data())
                return;
	else
		data()->setXY(x(), y());

	updateLink();
}

void DCanvasItemBase::updateLink()
{
        if ( data()->Parent() != -1 )
        {
                DCanvasItemBase* item = m_canvasview->canvasItem( data()->Parent() );

                if (!item)
		{
                        //// kdWarning()<<"bug in DCanvasItemBase::move"<<endl;
			return;
		}

                m_link->setPoints(
				x()+width()/2, 
				y()+height()/2, 
				item->x()+item->width()/2, 
				item->y()+item->height()/2 );
        }
        else
        {
                m_link->setPoints(0, 0, 0, 0);
        }
}

void DCanvasItemBase::updatePos()
{
	if (data())
        	move(data()->x(), data()->y());
	else
		move(x(), y());
        m_pos->updatePos();
	update();
}


void DCanvasItemBase::loadPos()
{
	move(data()->x(), data()->y());
}

bool DCanvasItemBase::onWidget(const QPoint & p)
{
	return ( x() <= p.x() && x() + width() >= p.x() &&
			y() <= p.y() && y() + height() >= p.y() );
}

void DCanvasItemBase::drawShape(QPainter &p)
{
	if (!data())
		return;

	drawContent(p);
	if (isSelected())
		drawSelected(p);
}

void DCanvasItemBase::drawContent(QPainter &p)
{
	if (!data())
		return;

	p.fillRect(rect(), QBrush(m_color) );
}

void DCanvasItemBase::drawSelected(QPainter& p)
{
	// draw 4 small squares, meaning that the object is selected
	int w = width();
	int h = height();
	int s = 4;

	QBrush brush(blue);
	p.fillRect((int) x()        , (int) y(),         (int) s, s, brush);
	p.fillRect((int) x()        , (int) y() + h - s, (int) s, s, brush);
	p.fillRect((int) x() + w - s, (int) y(),         (int) s, s, brush);
	p.fillRect((int) x() + w - s, (int) y() + h - s, (int) s, s, brush);
}

/*
DDataItemBase* DCanvasItemBase::data()
{
	if (!m_canvasview)
	{
		// kdWarning()<<"bug in DCanvasItemBase::data"<<endl;
		return NULL;
	}

	return m_canvasview->dataItem(Id());
}
*/

bool DCanvasItemBase::displayChildIdx() const
{
	return m_displaychildidx;
}

void DCanvasItemBase::setDisplayIdx(bool b)
{
	m_displaychildidx = b;
	m_pos->setVisible(b);
}

int DCanvasItemBase::numChild()
{
	if (data()->Parent() == -1)
		return -1;

	return m_canvasview->dataItem(data()->Parent())->childIdx(Id());
}

void DCanvasItemBase::setSize( int w, int h )
{
	if (width() != w || height() != h)
	{
		// kdWarning()<<"DCanvasItemBase::setSize true"<<endl;

		QCanvasRectangle::setSize( w, h );
		updatePos();
		m_canvasview->dataTree()->notifyChildren(Id());
	}
}


