/***************************************************************************
 *   Copyright (C) 2004 by Thomas Nagy                                     *
 *   tnagy256@yahoo.fr                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <qfont.h>
#include <qcolor.h>
#include <qstring.h>
#include <qstringlist.h>
#include <qpixmap.h>
#include <qvaluelist.h>
#include <qtextstream.h>
#include <qfile.h>
#include <qregexp.h>

#include <karchive.h>
#include <ktar.h>
#include <klocale.h>
#include <ktempfile.h>
#include <ktempdir.h>
#include <kmessagebox.h>
#include <kio/netaccess.h>
#include <kurl.h>
#include <kdebug.h>

#include "constants.h"
#include "../itemconfig/ItemCfg.h"
#include "../gui/settings.h"
#include "DBase.h"
#include "DDataItemBase.h"
#include "DDataControl.h"
#include "DissertParser.h"

#include "DDataItemText.h"
#include "DDataItemPix.h"
#include "DDataItemRoot.h"


DDataControl::DDataControl(QObject * parent, const char * name) : QObject(parent, name)
{
	m_lastid = 0;
}

DDataControl::~DDataControl()
{

}

DDataItemBase* DDataControl::createItem(DItem::itype type, int id)
{
	DDataItemBase* item = NULL;

	//
	if (isRegistered(id))
	{
		kdWarning()<<"attempted to create an item that already exists!"<<endl;
		return NULL;
	}
	switch (type)
	{
		case DDataItemBase::Text:
			item = new DDataItemText(id);
			break;

		case DDataItemBase::Pix:
			item = new DDataItemPix(id);
			break;

		case DDataItemBase::Root:
			item = new DDataItemRoot(id);
			break;

		default:
			break;
	}

	if (item)
		registerItem(item);

	return item;
}

void DDataControl::createItem(DItem::itype type)
{
	DDataItemBase * item = NULL;

	int count = m_lastid;
	count++;

	// kdWarning()<<"DDataControl::createItem "<<Total()<<endl;

	while ( isRegistered(count) )
	{
		count++;
	}

	item = createItem(type, count);
	if (item)
	{
		emit itemCreated( item->Id() );
	}

	if ( Settings::popupDlgCreate() )
	{
		configure( item->Id() );
	}
}

void DDataControl::linkItems(int id1, int id2)
{
	// kdWarning()<<"call to DDataControl::linkItems"<<endl;

	if (id1 == -1 || id2 == -1)
		return;

	if (id1 == id2)
		return;

	bool changed = false;

	// one item must have a parent;
	if ( dataItem(id1)->Parent() !=- 1 && dataItem(id2)->Parent() == -1 )
	{
		changed = true;
		dataItem(id1)->addChild(id2);
		dataItem(id2)->setParent(id1);
	}
	else if ( dataItem(id2)->Parent() != -1 && dataItem(id1)->Parent() == -1)
	{
		changed = true;
		dataItem(id2)->addChild(id1);
		dataItem(id1)->setParent(id2);
	}

	if (changed)
	{
		emit itemChanged(id1);
		emit itemChanged(id2);
	}
}

void DDataControl::unlinkItems(int id1, int id2)
{

	//kdWarning()<<"unlink "<<id1<<" "<<id2<<endl;

	if (id1 == -1 || id2 == -1)
		return;

	if (id1 == id2)
		return;

	bool changed = false;


	if (!dataItem(id1))
	{
		dataItem(id2)->removeChild(id1);
	}
	else if (!dataItem(id2))
	{
		dataItem(id1)->removeChild(id2);
	}
	else if (dataItem(id1)->Parent() == id2)
	{
		changed = true;
		dataItem(id1)->setParent(-1);
		dataItem(id2)->removeChild(id1);
	}
	else if (dataItem(id2)->Parent() == id1)
	{
		changed = true;
		dataItem(id2)->setParent(-1);
		dataItem(id1)->removeChild(id2);
	}

	if (changed)
	{
		emit itemChanged(id1);
		emit itemChanged(id2);
	}
}

void DDataControl::configure(int id)
{
	ItemCfg::config( dataItem(id) );
	emit itemChanged( id );
	notifyChildren( id );
}

void DDataControl::removeItem(int id)
{
	if (!isRegistered(id) )
		return;

	disconnectItem(id);
	unregisterItem(id);
	emit itemRemoved( id );
}

void DDataControl::disconnectItem(int id)
{
	if (!isRegistered(id))
		return;

	killChildren(id);
	setOrphan(id);
}

void DDataControl::setOrphan(int child)
{
	if (!isRegistered(child))
		return;
	
	if (!child == -1)
		return;
	int parent = dataItem(child)->Parent();

	if (parent == -1)
		return;

	if (child == parent)
		return;

	unlinkItems(child, parent);
}

void DDataControl::killChildren(int id)
{
	if (!isRegistered(id))
		return;

	DDataItemBase* item = dataItem(id);
	while (item->countChildren()>0)
	{
		unlinkItems(id, item->childNum(0));
	}
}

void DDataControl::killFamily(int id)
{
	while (dataItem(id)->countChildren() > 0)
	{
		killFamily( dataItem(id)->childNum(0) );
	}
	setOrphan(id);
}

DDataItemBase* DDataControl::dataItem(int id)
{
	return (DDataItemBase*) DBase::Item(id);
}

void DDataControl::clearDocument()
{
	checkConsistency();

	QValueList<int>::iterator it;
	QValueList<int> allitems = m_map.keys();
	for ( it = allitems.begin(); it != allitems.end(); ++it )
	{
		removeItem(*it);
	}
}

void DDataControl::notifyChildren(int id)
{
	DDataItemBase * item = dataItem(id);
	if (item)
	{
		// update the children positions
		for (int i = 0; i<item->countChildren(); i++)
		{
			emit itemChanged(item->childNum(i));
		}
	}
}

void DDataControl::printTree(QTextStream & s)
{
	s.setEncoding( QTextStream::UnicodeUTF8 );

	s<<QString("<?xml version=\"1.0\" encoding=\"utf8\"?>\n");
	s<<"<kdissertdoc>\n";

	QMapIterator<int, DItem*> it;
	for (it = m_map.begin(); it != m_map.end(); ++it)
	{
		DDataItemBase* item = (DDataItemBase*) (*it);
		if (item)
			item->printSelf(s);
		else
			kdWarning()<<"BUG : in DDataControl::printTree"<<endl;
	}
	s<<"</kdissertdoc>\n";
}

bool DDataControl::loadFromFile(const KURL & url)
{
	clearDocument();

	QString tmpFile;
	if (! KIO::NetAccess::download(url, tmpFile, NULL))
	{
		QString text;

		if ( url.isLocalFile() )
			text = i18n( "Unable to find the kdissert file %1 !" );
		else
			text = i18n( "Unable to download the kdissert file ! "
					"Please check that the address %1 is correct." );

		KMessageBox::sorry( NULL, text.arg( url.prettyURL() )  );
		return false;
	}

	KTar arch(tmpFile);
	if (! arch.open(IO_ReadOnly) )
	{
		QString text = i18n("Unable to open the kdissert file %1 !");
		KMessageBox::sorry( NULL, text.arg( url.prettyURL() )  );
		KIO::NetAccess::removeTempFile( tmpFile );
		return false;
	}

	const KArchiveDirectory *archiveDir = arch.directory();
	QStringList lst = archiveDir->entries();

	// Find the main document or return false
	bool cond = false;
	for (unsigned int i=0; i<lst.count(); i++)
	{
		if (lst[i] == "maindoc.xml")
			cond = true;
	}

	if (!cond)
	{
		QString text = i18n("Unable to parse the kdissert file %1 !");
		KMessageBox::sorry( NULL, text.arg( url.prettyURL() )  );
		KIO::NetAccess::removeTempFile( tmpFile );
		return false;
	}

	const KArchiveEntry* entry = archiveDir->entry("maindoc.xml");
	if (!entry->isFile())
	{
		QString text = i18n("The kdissert file %1 does not seem to be a valid kdissert file !");
		KMessageBox::sorry( NULL, text.arg( url.prettyURL() )  );
		KIO::NetAccess::removeTempFile( tmpFile );
		return false;
	}

	KArchiveFile* maindocentryfile = (KArchiveFile*) entry;
	KTempDir tmpdir;
	maindocentryfile->copyTo(tmpdir.name());

	// Parse the maindoc file now
	QString tmpdocname = tmpdir.name() + maindocentryfile->name();
	QFile maindocfile(tmpdocname);
	DissertParser handler(this);
	QXmlInputSource source( &maindocfile );
	QXmlSimpleReader reader;
	reader.setContentHandler( &handler );

	bool result = reader.parse( &source , true );
	if (!result)
	{
		QString text = i18n("The kdissert file %1 does not seem to be a valid kdissert file !");
		KMessageBox::sorry( NULL, text.arg( url.prettyURL() )  );
		clearDocument();
		KIO::NetAccess::removeTempFile( tmpFile );
		tmpdir.unlink();
		return false;
	}

	// Load the pictures
	QRegExp rx("^pic.\\d+.png$");
	for (unsigned int i=0; i<lst.count(); i++)
	{
		if (rx.search(lst[i]) == 0)
		{
			int id = lst[i].section(".", 1, 1).toInt();
			// kdWarning()<<"file is "<<lst[i]<<" and id is : "<<id<<endl;
			DDataItemPix* pixitem = (DDataItemPix*) dataItem(id);
			if (pixitem)
			{
				entry = archiveDir->entry(lst[i]);
				if (entry->isFile())
				{
					KArchiveFile* picfile = (KArchiveFile*) entry;
					picfile->copyTo(tmpdir.name());
					pixitem->loadPix(tmpdir.name() + lst[i]);
				}
			}
		}
	}


	// Notify the views attached
	QValueList<int>::iterator it;
	QValueList<int> allitems = m_map.keys();
	for ( it = allitems.begin(); it != allitems.end(); ++it )
	{
		emit itemCreated(*it);
	}

	// make sure everything is correct (perhaps a bit paranoid ..)
	for ( it = allitems.begin(); it != allitems.end(); ++it )
	{
		emit itemChanged(*it);
	}	

	KIO::NetAccess::removeTempFile( tmpFile );
	tmpdir.unlink();

	return result;
}

bool DDataControl::saveToFile(const KURL & url)
{
	KTempFile tf;
	tf.close();
	tf.setAutoDelete(true);

	KTar arch(tf.name(), "application/x-gzip");
	//QFile file(tf.name());
	if (! arch.open(IO_WriteOnly) )
	{
		KMessageBox::information(NULL, i18n("Cannot open file '%1' for output.").arg(tf.name()), i18n("Save Document") );
		return false;
	}

	// add the doc file
	KTempFile docfile;
	docfile.setAutoDelete(true);
	printTree(*docfile.textStream());
	docfile.close();
	arch.addLocalFile(docfile.name(), "maindoc.xml");

	// add the pictures
	QValueList<int>::iterator it;
	QValueList<int> allitems = m_map.keys();
	for ( it = allitems.begin(); it != allitems.end(); ++it )
	{
		DDataItemBase* item = (DDataItemBase*) dataItem(*it);
		if (item->Type() == DItem::Pix)
		{
			DDataItemPix* pix = (DDataItemPix*) item;
			KTempFile picfile;
			picfile.close();
			picfile.setAutoDelete(true);

			// png is a lossless compresion scheme, right ?
			pix->m_pix.save(picfile.name(), "PNG", 0 );
			arch.addLocalFile(picfile.name(), QString("pic.%1.png").arg(pix->Id()) );
		}
	}

	// close the archive and save it
	arch.close();
	KIO::NetAccess::upload(tf.name(), url, NULL);

	return true;
}

int DDataControl::countItems()
{
	return m_map.count();
}

int DDataControl::idNum(int num)
{
	if (num < countItems())
	{
		return m_map.keys()[num];
	}
	return 0;
}

bool DDataControl::canGenerate()
{
	DDataItemBase *data = dataItem( rootID() );

	if (!data)
		return false;

	if (data->countChildren() > 0)
	{
		return true;
	}

	return false;
}

int DDataControl::rootID()
{
	QValueList<int>::iterator it;
	QValueList<int> allitems = m_map.keys();
	for ( it = allitems.begin(); it != allitems.end(); ++it )
	{
		if ( dataItem( *it )->Type() == DItem::Root)
		{
			return *it;
		}
	}
	return NOITEM;
}

void DDataControl::debugItem(int id)
{
	DDataItemBase *item = dataItem( id );
	if (!item)
	{
		kdWarning()<<"DDataControl::debugItem : invalid item of id : "<<id<<endl;
		return;
	}
	kdWarning()<<"***"<<endl;

	kdWarning()<<"item of id : "<<id<<endl;

	for (int i=0; i<item->countChildren(); i++)
	{
		kdWarning()<<"   - child "<<dataItem(item->childNum(i))->Id()<<endl;
	}

	kdWarning()<<"***"<<endl;
}

void DDataControl::checkConsistency()
{
	// TODO  double-check
	// 
        QValueList<int>::iterator it;
        QValueList<int> allitems = m_map.keys();
        for ( it = allitems.begin(); it != allitems.end(); ++it )
        {
		bool changed = false;

		
		// check parent
                DDataItemBase* item = (DDataItemBase*) dataItem(*it);
		if (item->Parent() != NOITEM)
		{
			if (!dataItem(item->Parent()))
			{
				item->setParent(NOITEM);
				changed = true;	
			}
		}
		
		// check children
		int count = 0;
		while (count < item->countChildren())
		{
			if (!dataItem(item->childNum(count)))
			{
				item->removeChild( item->childNum(count) );
				count = 0;
				changed = true;
			}
			else
			{
				count++;
			}
		}
		
		// notify if something changed
		if (changed)
		{
			kdWarning()<<"inconsistency detected - item : "<<item->Id()<<endl;	
			emit itemChanged(item->Id());
		}
	}
}

#include "DDataControl.moc"

