/***************************************************************************
 *   Copyright (C) 2004 by Thomas Nagy                                     *
 *   tnagy256@yahoo.fr                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <qlabel.h>
#include <qlayout.h>
#include <qvaluelist.h>
#include <qpainter.h>
#include <qcanvas.h>
#include <qpixmap.h>

#include <klocale.h>
#include <kmessagebox.h>
#include <kstandarddirs.h>
#include <kdebug.h>
#include <klibloader.h>
#include <ktempfile.h>

#include <kfiledialog.h>
#include <kio/netaccess.h>
#include <kurl.h>

#include "../canvasview/DCanvasView.h"
#include "../treelistview/DTreeListView.h"
#include "kdissertview.h"
#include "settings.h"

#include "generatorwizard.h"

#include "../datastruct/DDataItemBase.h"
#include "../datastruct/DDataControl.h"
#include "../generator/DGenerator.h"


kdissertView::kdissertView(QWidget *parent)
    : kdissertview_base(parent)
{
	m_data = new DDataControl();
	
	m_canvasview = new DCanvasView(canvasframe);
	m_canvasview->setDataTree(m_data);
	m_canvasview->setFocusPolicy( QWidget::StrongFocus );
	m_canvasview->setFocus();
	
	QGridLayout *l = new QGridLayout(canvasframe, 1, 1);
	l->addWidget(m_canvasview, 0, 0);


	m_treelistview = new DTreeListView(treelistviewframe);
	m_treelistview->setDataTree(m_data);
	
	l = new QGridLayout(treelistviewframe, 1, 1);
	l->addWidget(m_treelistview, 0, 0);
	
	
	settingsChanged();
}

kdissertView::~kdissertView()
{

}

void kdissertView::settingsChanged()
{
	// i18n : internationalization
	emit signalChangeStatusbar( i18n("Settings changed") );
	m_canvasview->settingsChanged();
	m_treelistview->settingsChanged();
}

void kdissertView::setLinkMode()
{
	m_canvasview->setMode( DCanvasView::act_link );
}

void kdissertView::setPointMode()
{
	m_canvasview->setMode( DCanvasView::act_point );
}

void kdissertView::setSortMode()
{
	m_canvasview->setMode( DCanvasView::act_sort );
}

void kdissertView::clearDocument()
{
	if (!m_data)
		return;

	int result = KMessageBox::warningContinueCancel(
			this,
			i18n("You are about to clear the entire document. Are you sure ?"),
			i18n("Clear the entire document"),
			i18n("Clear"));

	if (result == KMessageBox::Continue)
		m_data->clearDocument();
}

bool kdissertView::saveToFile(const KURL & url)
{
	if (!m_data)
		return false;
	return m_data->saveToFile(url);
}

bool kdissertView::loadFromFile(const KURL & url)
{
	if (!m_data)
		return false;
	return m_data->loadFromFile(url);
}

void kdissertView::generateDocument()
{
	if (!m_data)
		return;
	
	if (!m_data->canGenerate())
	{
		KMessageBox::sorry(this, 
				i18n("Your document does not have any root connected to children\n"
				       "kdissert will not generate any document"), 
				i18n("Invalid document") );
		return;
	}


	QStringList lst = KGlobal::instance()->dirs()->findAllResources("lib", "*kdiss*.la");
	// kdWarning()<<" *** kdissert generators found *** \n"<<lst.join("\n")<<endl;

	// load the libraries
	QValueList<DGenerator*> liblist;
	for (unsigned int i=0; i<lst.count(); i++)
	{
		QString libname = lst[i].section( '/', -1 );
		libname.truncate(libname.length()-3);
		// kdWarning()<<"loading : "<<libname<<endl;

		KLibFactory* factory = KLibLoader::self()->factory(libname);
		if (factory)
		{
			 //kdWarning()<<"library successfully dlopened!!"<<endl;

			DGenerator* lib = static_cast<DGenerator*> (factory->create(this, "hi", "ho"));
			 //kdWarning()<<"library full name is  : "<<lib->fullName()<<endl;
			 //kdWarning()<<"library group is : "<<lib->group()<<endl;
			liblist.push_back(lib);
		}
		else
		{
			 kdWarning()<<"library "<<lst[i]<<" is invalid "<<KLibLoader::self()->lastErrorMessage()<<endl;
		}
	}

	if (liblist.count() > 0)
	{
		generatorwizard *wiz = new generatorwizard(this, "generatorwizard");
		wiz->setgeneratorlist(liblist);
		wiz->setgeneratordata(m_data);
		wiz->exec();
	}
	else
	{
		KMessageBox::sorry(this, i18n("No templates were found, please check your installation"), i18n("Templates are missing") );
	}
	
	// clean after use
	for (unsigned int i=0; i<liblist.count(); i++)
	{
		delete liblist[i];
	}
	KLibLoader::self()->cleanUp();
}

void kdissertView::savePic()
{
	if (!m_data)
	{
		return;
	}

	KURL url = KFileDialog::getSaveURL(QString::null, i18n("*.png|Png files (*.png)"), this, i18n("Grab picture of mindmap"));
	
	if (url.isEmpty() || !url.isValid())
	{
		//kdWarning()<<"invalid url !"<<endl;
		return;
	}

	QCanvas * canvas = m_canvasview->canvas();
        QPixmap pix(canvas->width(), canvas->height());
        QPainter p(&pix);
        canvas->drawArea( canvas->rect(), &p );

	KTempFile tf;
	tf.close();
	tf.setAutoDelete(true);
	
        pix.save(tf.name(), "PNG");
	KIO::NetAccess::upload(tf.name(), url, NULL);
}

#include "kdissertview.moc"

