/***************************************************************************
 *   Copyright (C) 2004 by Thomas Nagy                                     *
 *   tnagy256@yahoo.fr                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <qevent.h>
#include <qcolor.h>
#include <qdragobject.h>

#include <kdebug.h>
#include <klocale.h>

#include "DTreeListView.h"
#include "DTreeListItemBase.h"
#include "DTreeListItemRoot.h"
#include "DTreeListItemText.h"
#include "DTreeListItemPix.h"

#include "../datastruct/DDataControl.h"
#include "../datastruct/constants.h"

DTreeListView::DTreeListView( QWidget* parent, const char* ) : KListView(parent), DGuiView(parent)
{
	addColumn(i18n("Concept list"));
	setResizeMode(QListView::AllColumns);
	setSorting(0);


	setDragEnabled(true);
	setAcceptDrops(true);

	setItemsMovable(true);
	setDropVisualizer(true);

	setSelectionModeExt( KListView::Single );

	setRootIsDecorated(true);

	connect( this, SIGNAL(doubleClicked ( QListViewItem *, const QPoint &, int )), 
			this, SLOT(configureItem(QListViewItem *, const QPoint &, int ))  );

	connect(
	this, SIGNAL( moved( QListViewItem *, QListViewItem *, QListViewItem * ) ),
	this, SLOT( applyMoved( QListViewItem *, QListViewItem * , QListViewItem * ) ) 
	       );

}

DTreeListView::~DTreeListView()
{

}

void DTreeListView::settingsChanged()
{
	//kdWarning()<<"DTreeListView::settingsChanged"<<endl;
}

void DTreeListView::createItem( int id )
{
	//kdWarning()<<"DTreeListView::createItem"<<endl;

	if (!dataItem(id))
	{
		kdWarning()<<"Bug: no item given in DTreeListView::createItem"<<endl;
		return;
	}

	DTreeListItemBase *item = NULL;
	switch (dataItem(id)->Type())
	{
		case DItem::Text:
			item = new DTreeListItemText(this, id);
			break;
		case DItem::Root:
			item = new DTreeListItemRoot(this, id);
			break;
		case DItem::Pix:
			item = new DTreeListItemPix(this, id);
			break;
		default:
			break;
	}

	if (item)
		registerItem(item);
}

void DTreeListView::updateItem(int id)
{
	//kdWarning()<<"DTreeListView::updateItem"<<endl;
	DTreeListItemBase *item = (DTreeListItemBase*) Item(id);
	if (!item)
		return;

	int parentid = dataItem(id)->Parent();
	if (parentid != id)
	{
		DTreeListItemBase *child = dynamic_cast<DTreeListItemBase*> (Item(id));
		DTreeListItemBase *currentparent = dynamic_cast<DTreeListItemBase*> (child->parent());
		DTreeListItemBase *newparent= dynamic_cast<DTreeListItemBase*> (Item(parentid));

		if (newparent != currentparent)
		{
			if (currentparent)
				currentparent->takeItem(child);
			else
				takeItem(child);

			if (parentid == NOITEM)
				insertItem(child);
			else
				newparent->insertItem(child);
		}
	}

	item->update();
	sort();
}

void DTreeListView::sort()
{

}

void DTreeListView::removeItem(int id)
{
	//kdWarning()<<"DTreeListView::removeItem"<<endl;
	unregisterItem(id);
	DTreeListItemBase *item = (DTreeListItemBase*) Item(id);
	if (!item)
		return;
	delete item;
	//takeItem(item);
}

void DTreeListView::configureItem(QListViewItem * it, const QPoint &, int )
{
	DTreeListItemBase *item = (DTreeListItemBase*) it;
	if (!item)
		return;

	dataTree()->configure(item->Id());
}

void DTreeListView::plug()
{
	// kdWarning()<<"DTreeListView::plug"<<endl;
	if (!dataTree())
		return;
	connect((QObject*) dataTree(), SIGNAL( itemChanged(int) ), this, SLOT( updateItem(int) ) );
	connect((QObject*) dataTree(), SIGNAL( itemCreated(int) ), this, SLOT( createItem(int) ) );
	connect((QObject*) dataTree(), SIGNAL( itemRemoved(int) ), this, SLOT( removeItem(int) ) );
}

void DTreeListView::unplug()
{
	// kdWarning()<<"DTreeListView::unplug"<<endl;
	if (!dataTree())
		return;
	disconnect((QObject*) dataTree(), SIGNAL( itemChanged(int) ), this, SLOT( updateItem(int) ) );
	disconnect((QObject*) dataTree(), SIGNAL( itemCreated(int) ), this, SLOT( createItem(int) ) );
	disconnect((QObject*) dataTree(), SIGNAL( itemRemoved(int) ), this, SLOT( removeItem(int) ) );
}

void DTreeListView::keyPressEvent(QKeyEvent *e)
{
	disconnect(
	this, SIGNAL( moved( QListViewItem *, QListViewItem *, QListViewItem * ) ),
	this, SLOT( applyMoved( QListViewItem *, QListViewItem * , QListViewItem * ) ) 
	       );


	DTreeListItemBase *item = (DTreeListItemBase*) selectedItem();
	int id = item->Id();

	if (e->key() == Qt::Key_Delete)
	{
		if (item)
		{
			clearSelection();
			dataTree()->removeItem( id );
		}
	}
	else if (e->key() == Qt::Key_D)
	{
		//kdWarning()<<"demote !!"<<endl;
		if (item)
		{
			DTreeListItemBase *parent = (DTreeListItemBase*) item->itemAbove();
			if (parent)
			{
				dataTree()->setOrphan(id);
				dataTree()->linkItems(id, parent->Id());
				updateItem(id);
			}
		}
	}
	else if (e->key() == Qt::Key_P)
	{
		//kdWarning()<<"promote !!"<<endl;
		if (item)
		{
			int parent = dataItem(id)->Parent();

			// 
			dataTree()->setOrphan(id);
		
			// now link the item to the parent of its parent
			if (parent != NOITEM)
			{
				dataTree()->linkItems(id, dataItem( parent )->Parent() );
			}
		}
	}

	connect(
	this, SIGNAL( moved( QListViewItem *, QListViewItem *, QListViewItem * ) ),
	this, SLOT( applyMoved( QListViewItem *, QListViewItem * , QListViewItem * ) ) 
	       );
}

void DTreeListView::applyMoved( QListViewItem* current, QListViewItem*, QListViewItem* )
{
	if (!current)
		return;

	DTreeListItemBase *item = NULL;
	int idcur = NOITEM, idparent = NOITEM;

	item = (DTreeListItemBase*) current;
	if (item)
		idcur = item->Id();

	item = (DTreeListItemBase*) current->parent();
	if (item)
		idparent = item->Id();


	dataTree()->setOrphan( idcur );

	if (idparent != NOITEM)
		dataTree()->linkItems( idparent, idcur);

	updateItem( idcur );

	//kdWarning()<<"cur : "<<idcur<<" || parent : "<<idparent<<endl;
}


#include "DTreeListView.moc"

